<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       http://hyperxmedia.com
 * @since      1.0.0
 *
 * @package    Marketo_forms
 * @subpackage Marketo_forms/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Marketo_forms
 * @subpackage Marketo_forms/public
 * @author     HyperX Media <sarmstrong@hyperxmedia.com>
 */
class Marketo_forms_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Marketo_forms_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Marketo_forms_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/ult_marketo_forms-public.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		wp_enqueue_script( 'mrktofrms-1', plugin_dir_url( __FILE__ ) . 'js/forms2.min.js', array( 'jquery' ), $this->version, false );
		wp_enqueue_script( 'mrktofrms-2', plugin_dir_url( __FILE__ ) . 'js/ult_marketo_forms-public.js', array( 'jquery' ), $this->version, false );
	}


	public function register_shortcodes(){
		add_shortcode( 'mrkto_frm', array( $this, 'mrkto_frm_func') );
	}



	public function mrkto_frm_func( $atts ) {
		global $wpdb;

		if( !is_numeric($atts['id']) ) die('marketo id needs to be a number');

		//$api_host = get_option('ult_marketo_forms_host');
		$plugin_path = admin_url( 'admin-ajax.php' );
		$db_form = $wpdb->get_row( 'SELECT * FROM '.$wpdb->prefix . 'ult_marketo_forms WHERE form_id = "'.esc_sql($atts['id']).'"');

		// Style
		if($db_form->style_id==-1){
			$style = "<style>
/* Override Layout */

.umfwrap .mktoForm,
.umfwrap .mktoForm * {
	box-sizing: border-box;
	padding:0px;
}

.umfwrap .mktoForm .mktoRequiredField .mktoAsterix,
.umfwrap .mktoForm .mktoAsterix,
.umfwrap .mktoForm .mktoClear,
.umfwrap .mktoForm .mktoGutter,
.umfwrap .mktoForm .mktoOffset,
.umfwrap .mktoForm .mktoFormRow.hidden {
	display: none;
}

.umfwrap .mktoHasWidth {
	width: 100% !important;
}

.umfwrap .mktoButtonRow,
.umfwrap .mktoForm,
.umfwrap .mktoFormCol,
.umfwrap .mktoField
.umfwrap .mktoFieldWrap {
	width: 100%;
}

.umfwrap .mktoForm .mktoFieldWrap {
	float: none;
}

.umfwrap .mktoFormCol {
	margin-bottom: 1em !important;
}

/* Two Column Rows */
.umfwrap .mktoForm .mktoFormRow {
	display: flex;
}

.umfwrap .mktoFormCol:nth-child(2){
	margin-left: 10px;
}

.umfwrap .mktoFormCol:nth-last-child(3) {
	margin-right: 10px;
}

/* Label */

.umfwrap .mktoForm label {
	font-family: Helvetica, Arial, sans-serif;
	margin-bottom: 5px;
	padding-top: 0;
	font-size: 1em;
}

/* Inputs */

.umfwrap .mktoForm input,
.umfwrap .mktoForm input[type=text],
.umfwrap .mktoForm input[type=url],
.umfwrap .mktoForm input[type=email],
.umfwrap .mktoForm input[type=tel],
.umfwrap .mktoForm input[type=number],
.umfwrap .mktoForm input[type=date],
.umfwrap .mktoForm textarea.mktoField,
.umfwrap .mktoForm select.mktoField {
	font-family: Helvetica, Arial, sans-serif;
	font-size: 1em;
	height: auto;
	padding: 8px 8px;
	color: #333;
	background-color: white;
	border: 1px solid #ccc;
}

.umfwrap .mktoForm select.mktoField {
	height: 33px;
}


/* Submit Button */

.umfwrap .mktoForm .mktoButtonRow .mktoButtonWrap {
	margin-left: 0px !important;
}

.umfwrap .mktoForm .mktoButtonRow .mktoButtonWrap button.mktoButton {
	font-family: Helvetica, Arial, sans-serif;
	background-image: none;
	color: #fff;
	background-color: #0A95B7;
	font-size: 1em;
	border: none;
	padding: 11px 24px 9px 24px;
	text-transform: uppercase;
}
.umfwrap .mktoForm .mktoButtonRow .mktoButtonWrap button.mktoButton:hover {
	background-color: #16819e;
}
</style>";
		} else if($db_form->style_id>0){
			$styling = $wpdb->get_row( 'SELECT * FROM '.$wpdb->prefix . 'ult_marketo_forms_styles WHERE id = "'.$db_form->style_id.'"');
			$style = '<style>'.esc_html($styling->style).'</style>';
		} else $style = '';


		$js_options = $url_vars = $ty_js = '';
		if(!empty($db_form)){
			// Hidden Fields
			if($db_form->data_options>''){
				$options = json_decode($db_form->data_options);
				for ($i = 0; $i < count($options->location); $i++) {
					if($options->location[$i] == 'url'){
						$js_options .= '<input type="hidden" name="' . esc_attr($options->mrkto_field[$i]) . '" id="' . esc_attr($options->mrkto_field[$i]) . '" value="">';
						$url_vars .= 'jQuery("#mktoForm_"+form_id+" #' . esc_attr($options->mrkto_field[$i]) . '").val( get_url_var(\'' . esc_attr($options->var_name[$i]) . '\') );'."\r\n				";
					}
				}
			}

			// Thank you page
			if($db_form->ty_options>''){
				$operations = array(
					'equals' => '==',
					'greater' => '>',
					'less' => '<',
					'not_equal' => '!='
				);
				$options = json_decode($db_form->ty_options);
				$checks = array();
				$cur_matching = true;
				for($i = 0; $i < count($options->field); $i++){
					if($options->field[$i] == 'ty_url'){
						if($cur_matching){
							$ty_js.= "					if(".implode(' && ', $checks).") window.location = '".esc_html($options->ty_url[$i])."';\r\n";
						}
						$checks = array();
						$cur_matching = true;
					} else {
						if(is_numeric($options->value[$i])) $val = $options->value[$i];
						else $val = '"'.$options->value[$i].'"';
						$checks[] = 'values.'.$options->field[$i].' '.$operations[$options->operation[$i]].' '.$val;
					}
				}
				$ty_js.= '					return false;';
			} else $ty_js = '					return true;';
		}

		$api_host = get_option('ult_marketo_forms_host');
		$account_id = str_replace(array('https://', 'http://'), '', $api_host);
		$account_id = explode('.', $account_id);
		$account_id = $account_id[0];




		// Vars for HTML
		$atts_id = esc_html($atts['id']);
		$host_link = esc_html($db_form->host_link);
		$account_id = esc_html($account_id);





		// HTML For Form
		$html = <<<HTML
		<div class="umfwrap">
		<form id="mktoForm_{$atts_id}"></form>
		</div>
		{$style} 
		<script>
			if(!plugin_path) var plugin_path = '{$plugin_path}';
			MktoForms2.loadForm("{$host_link}", "{$account_id}", {$atts_id}, function(form) {
				// Add an onSuccess handler
				form.onSuccess(function(values, followUpUrl) {
					// Get the form's jQuery element and hide it
					form.getFormElem().hide();
					{$ty_js}					
				});
			});
		</script>
		<div id="iframe_area"></div>



		<!-- NEEDED BUT NOT SURE IF SHOULD BE IN PLUGIN -->
		<script type="text/javascript">document.write(unescape("%3Cscript src='//munchkin.marketo.net/munchkin.js' type='text/javascript'%3E%3C/script%3E"));</script>
		<script>Munchkin.init('{$account_id}');</script>




		<script>
			function update_umrktofrm_{$atts_id}_fields(form_id){
				// update fields based on url vars
				jQuery("#mktoForm_"+form_id).append('{$js_options}');
				
				{$url_vars}
			}
		</script>
HTML;
		return $html;
	}


	private function getToken($api_host){
		$api_client_id = get_option('ult_marketo_forms_client_id');
		$api_client_secret = get_option('ult_marketo_forms_client_secret');

		if($api_host>'' && $api_client_id>'' && $api_client_secret>'') {
			$api_host_base = str_replace('/rest', '', $api_host);
			$ch = curl_init($api_host_base . "/identity/oauth/token?grant_type=client_credentials&client_id=" . $api_client_id . "&client_secret=" . $api_client_secret);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($ch, CURLOPT_HTTPHEADER, array('accept: application/json',));
			$response = json_decode(curl_exec($ch));
			curl_close($ch);


			$data = new stdClass();
			if(isset($response->access_token)) {
				$data->status = 'success';
				$data->token = $response->access_token;
			} else {
				$data->status = 'failed';
				$data->reason = $response->error_description;
			}
		}
		return $data;
	}






	public function ajax_check_mrkto_lead() {
		$api_host = get_option('ult_marketo_forms_host');
		$api_token = $this->getToken($api_host);

		$filter_val = sanitize_text_field($_POST['filterValues']);

		$url = $api_host.'/v1/leads.json?filterType=cookie&filterValues='.urlencode($filter_val).'&access_token='.$api_token->token;
		$ch = curl_init($url);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array('accept: application/json'));
		$response = json_decode(curl_exec($ch));
		curl_close($ch);


		if(empty($response->result)) {
			echo json_encode(array(
				'status' => 'failed',
				'reason' => 'no lead found'
			));
		} else if(!isset($response->result[0])){
			echo json_encode(array(
				'status' => 'failed',
				'reason' => print_r($response, true)
			));
		} else {
			echo json_encode(array(
				'status' => 'success',
				'info' => $response->result[0]
			));
		}

		wp_die();
	}
}
